@@ -45,7 +45,7 @@ impl Walue<usize> for CharSet {
     fn read<T: Tape>(tape: &mut T, glyphs: usize) -> Result<Self> {
         Ok(match try!(tape.peek::<u8>()) {
             0 => unimplemented!(),
-            1 => CharSet::Format1(read_walue!(tape, glyphs)),
+            1 => CharSet::Format1(try!(tape.take_given(glyphs))),
             2 => unimplemented!(),
             _ => raise!("found a char set with an unknown format"),
         })
@@ -62,7 +62,7 @@ impl Walue<usize> for Format1 {
         let mut ranges = vec![];
         let mut found = 0 + 1;
         while found < glyphs {
-            let range = read_value!(tape, Range1);
+            let range = try!(tape.take::<Range1>());
             found += 1 + range.left as usize;
             ranges.push(range);
         }
