@@ -4,6 +4,18 @@ use Result;
 
 /// A type that can read.
 pub trait Tape: Read + Seek + Sized {
+    /// Read a value.
+    #[inline(always)]
+    fn take<T: Value>(&mut self) -> Result<T> {
+        Value::read(self)
+    }
+
+    /// Read a value given a parameter.
+    #[inline(always)]
+    fn take_given<T: Walue<P>, P>(&mut self, parameter: P) -> Result<T> {
+        Walue::read(self, parameter)
+    }
+
     #[doc(hidden)]
     fn count(&mut self) -> Result<u64> {
         let current = try!(self.position());
@@ -31,12 +43,6 @@ pub trait Tape: Read + Seek + Sized {
     fn position(&mut self) -> Result<u64> {
         self.seek(SeekFrom::Current(0))
     }
-
-    #[doc(hidden)]
-    #[inline(always)]
-    fn take<T: Value>(&mut self) -> Result<T> {
-        Value::read(self)
-    }
 }
 
 /// A type that can be read.
@@ -45,7 +51,7 @@ pub trait Value: Sized {
     fn read<T: Tape>(&mut T) -> Result<Self>;
 }
 
-/// A type that can be read provided a parameter.
+/// A type that can be read given a parameter.
 pub trait Walue<P>: Sized {
     /// Read a value.
     fn read<T: Tape>(&mut T, P) -> Result<Self>;
@@ -54,48 +60,38 @@ pub trait Walue<P>: Sized {
 impl<T: Read + Seek> Tape for T {}
 
 macro_rules! read(
-    ($tape:ident, $count:expr, $buffer:expr) => (
-        if try!(::std::io::Read::read($tape, $buffer)) != $count {
-            return raise!("failed to read as much as needed");
-        }
-    );
     ($tape:ident, $size:expr) => (unsafe {
         let mut buffer: [u8; $size] = ::std::mem::uninitialized();
-        read!($tape, $size, &mut buffer);
+        try!(::std::io::Read::read($tape, &mut buffer));
         ::std::mem::transmute(buffer)
     });
 );
 
 macro_rules! value {
-    ([$kind:ident; $count:expr], 1) => (impl Value for [$kind; $count] {
-        #[inline]
-        fn read<T: Tape>(tape: &mut T) -> Result<Self> {
-            Ok(read!(tape, $count))
+    ([$kind:ident; $count:expr], 1) => (
+        impl Value for [$kind; $count] {
+            #[inline]
+            fn read<T: Tape>(tape: &mut T) -> Result<Self> {
+                Ok(read!(tape, $count))
+            }
         }
-    });
-    ($kind:ident, 1) => (impl Value for $kind {
-        #[inline]
-        fn read<T: Tape>(tape: &mut T) -> Result<Self> {
-            Ok(read!(tape, 1))
+    );
+    ($kind:ident, 1) => (
+        impl Value for $kind {
+            #[inline]
+            fn read<T: Tape>(tape: &mut T) -> Result<Self> {
+                Ok(read!(tape, 1))
+            }
         }
-    });
-    ($kind:ident, $size:expr) => (impl Value for $kind {
-        #[inline]
-        fn read<T: Tape>(tape: &mut T) -> Result<Self> {
-            Ok($kind::from_be(read!(tape, $size)))
+    );
+    ($kind:ident, $size:expr) => (
+        impl Value for $kind {
+            #[inline]
+            fn read<T: Tape>(tape: &mut T) -> Result<Self> {
+                Ok($kind::from_be(read!(tape, $size)))
+            }
         }
-    });
-}
-
-macro_rules! walue {
-    ($kind:ty, 1) => (impl Walue<usize> for $kind {
-        fn read<T: Tape>(tape: &mut T, count: usize) -> Result<Self> {
-            let mut values = Vec::with_capacity(count);
-            unsafe { values.set_len(count) };
-            read!(tape, count, &mut values);
-            Ok(values)
-        }
-    });
+    );
 }
 
 value!(u8, 1);
@@ -103,4 +99,17 @@ value!(u16, 2);
 value!(u32, 4);
 value!([u8; 3], 1);
 
+macro_rules! walue {
+    ($kind:ty, 1) => (
+        impl Walue<usize> for $kind {
+            fn read<T: Tape>(tape: &mut T, count: usize) -> Result<Self> {
+                let mut buffer = Vec::with_capacity(count);
+                unsafe { buffer.set_len(count) };
+                try!(::std::io::Read::read_exact(tape, &mut buffer));
+                Ok(buffer)
+            }
+        }
+    );
+}
+
 walue!(Vec<u8>, 1);
